from setuptools import setup, find_packages

setup(name='phrank',
      version='1.6',
      description='Measuring phenotype set similarity',
      url='https://bitbucket.org/bejerano/phrank/',
      author='Bejerano Lab',
      author_email='kjag@cs.stanford.edu',
      license='MIT',
      packages=find_packages(),
      zip_safe=False)
